// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.navigation.navigator.operation;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;

import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.application.CdmChangeEvent.Action;
import eu.etaxonomy.cdm.api.service.UpdateResult;
import eu.etaxonomy.cdm.api.service.config.TaxonDeletionConfigurator;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.operation.RemotingCdmUpdateOperation;

/**
 * @author cmathew
 * @date 22 Jun 2015
 *
 */
public class RemotingDeleteTaxonNodeOperation extends RemotingCdmUpdateOperation {

    private final static String LABEL = "Delete Taxon Node operation";

    private final Set<ITaxonTreeNode> treeNodes;
    private final TaxonDeletionConfigurator config;

    /**
     * @param label
     * @param action
     * @param source
     * @param async
     */
    public RemotingDeleteTaxonNodeOperation(Object source,
            boolean async,
            Set<ITaxonTreeNode> treeNodes,
            TaxonDeletionConfigurator config) {
        super(LABEL, Action.Delete, source, async);
        this.treeNodes = treeNodes;
        this.config = config;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmUpdateOperation#doUpdateExecute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
     */
    @Override
    protected UpdateResult doUpdateExecute(IProgressMonitor monitor, IAdaptable info) throws Exception {


        if(treeNodes.size() == 1) {
            // when single node this is either a taxon or a classification
            ITaxonTreeNode node = treeNodes.iterator().next();
            if(node instanceof TaxonNode) {
                return CdmApplicationState.getCurrentAppConfig().getTaxonNodeService().deleteTaxonNode(((TaxonNode)node).getUuid(), config);
            } else if(node instanceof Classification) {
                entityType = Classification.class;
                return CdmApplicationState.getCurrentAppConfig().getClassificationService().delete(((Classification)node).getUuid());
            }
        } else {
            // when list we assume all are taxon nodes
            Set<UUID> treeNodeUuids = new HashSet<UUID>();
            ITaxonTreeNode entity = null;
            for(ITaxonTreeNode treeNode : treeNodes) {
                if(entity == null) {
                    entity = treeNode;
                }
                treeNodeUuids.add(treeNode.getUuid());
            }

            return CdmApplicationState.getCurrentAppConfig().getTaxonNodeService().deleteTaxonNodes(treeNodeUuids, new TaxonDeletionConfigurator());
        }
        return null;
    }

}
